clear
clc
%close all

%%%%%%%%%%%%%%%%%%%%%%%%
%define the signal parameters
%%%%%%%%%%%%%%%%%%%%%%%%
N = 64;
sig = 0.01;

om = 2*pi*[0.0625,0.0875,0.25,0.285,0.33,0.35,0.37,0.39,0.41,0.43,0.45,0.47,0.49];
amp = [1;1;1;0.3;0.1;0.1;0.1;0.1;0.1;0.1;0.1;0.1;0.1];
pha = pi/4*ones(length(om),1);

% om=[pi/2];
% amp=[1];
% pha = [pi/4];

alp = amp.*exp(1i*pha);
w = sqrt(sig/2)*(randn(N,1)+1i*randn(N,1));
%generate new signal
s = exp(1i*(0:N-1)'*om+1i*pi/4)*alp+w;

wc = pi*(0:0.00025:1);
Mmax = floor(N/2);
mc = 1:Mmax;
L = Mmax;
%estimate the complex amplitudes
a_apes = apes(s,L,wc,'forback');
r_apes = abs(a_apes);
p_apes = angle(a_apes);
a_capon = asc(s,L,wc,'forback');
r_capon = abs(a_capon);
p_capon = angle(a_capon);
[am,rm,pm,sm,m_fmaasc] = maasc(s,Mmax,wc,'forward');
r_fmaasc = sum(rm.*m_fmaasc,1);
p_fmaasc = sum(pm.*m_fmaasc,1);
a_per = (exp(-1i*wc'*(0:N-1))*s)/N;
r_per = abs(a_per);
p_per = angle(a_per);
[am,rm,pm,sm,m_fbmaasc] = maasc(s,Mmax,wc,'forback');
r_fbmaasc = sum(rm.*m_fbmaasc,1);
p_fbmaasc = sum(pm.*m_fbmaasc,1);


%% Plots

%amplitude
figure(1)
r_true = [zeros(1,length(om));amp'];
subplot(2,3,1)
plot(wc,r_apes)
hold on
plot([om;om],r_true,'k--')
hold off
legend('apes')
grid
subplot(2,3,2)
plot(wc,r_capon)
hold on
plot([om;om],r_true,'k--')
hold off
legend('ASC')
grid
subplot(2,3,3)
plot(wc,r_fmaasc)
hold on
plot([om;om],r_true,'k--')
hold off
legend('f-MAASC')
grid
subplot(2,3,4)
plot(wc,r_per)
hold on
plot([om;om],r_true,'k--')
hold off
legend('fft')
grid
subplot(2,3,5)
plot(wc,r_fbmaasc)
hold on
plot([om;om],r_true,'k--')
hold off
legend('fb-MAASC')
grid
subplot(2,3,6)
plot(wc,[r_apes.';r_capon.';r_fmaasc;r_fbmaasc])
hold on
plot([om;om],r_true,'k--')
hold off
legend('APES','ASC','f-MAASC','fb-MAASC')
grid
%comparison
figure(2)
plot(wc,[r_per.';r_apes.';r_capon.';r_fmaasc;r_fbmaasc])
hold on
plot([om;om],r_true,'k--')
hold off
legend('FFT','APES','ASC','f-MAASC','fb-MAASC')
%model order
figure(3)
subplot(2,1,1)
if size(m_fmaasc,2) > 1
  contour(wc,mc,m_fmaasc)
else
  stem(mc,m_fmaasc)
end
hold on
plot([om;om],[zeros(1,length(om));mc(end)*ones(1,length(om))],'k--')
hold off
legend('f-MAASC')
subplot(2,1,2)
if size(m_fbmaasc,2) > 1
  contour(wc,mc,m_fbmaasc)
else
  stem(mc,m_fbmaasc)
end
hold on
plot([om;om],[zeros(1,length(om));mc(end)*ones(1,length(om))],'k--')
hold off
legend('fb-MAASC')

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% export the data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%model selection
 [~,idx_f_maasc] = max(m_fmaasc);
 [~,idx_fb_maasc] = max(m_fbmaasc);
 data = [wc',idx_f_maasc',idx_fb_maasc'];
 fid = fopen(['model_sel.table'], 'wt');
 fprintf(fid,'%%Selection of the most probable model\n');
 fprintf(fid,'w f_maasc fb_maasc\n');
 for ii=1:size(data,1)
   fprintf(fid,'%1.5f ',data(ii,:));
   fprintf(fid,'\n');
 end
 fclose(fid);

%zoom of spectrum
 data = [wc',r_per,r_capon,r_apes,r_fmaasc',r_fbmaasc'];
 fid = fopen(['zoom_spec.table'], 'wt');
 fprintf(fid,'%%Complex amplitude estimate\n');
 fprintf(fid,'w per fb_asc fb_apes f_maasc fb_maasc\n');
 for ii=1:size(data,1)
   fprintf(fid,'%1.5f ',data(ii,:));
   fprintf(fid,'\n');
 end
 fclose(fid);
