clear
clc
close all

%%%%%%%%%%%%%%%%%%%%%%%%
%define the signal parameters
%%%%%%%%%%%%%%%%%%%%%%%%
N = 64;
om = 2*pi*0.0625;
amp = 1;
phase = pi/4;
alp = amp*exp(1i*phase);
SNR = 0:2:40;
sig = amp^2*10.^(-SNR/10);
Nsnr = length(SNR);
%interfering sinusoids
omi = 2*pi*[0.0875,0.25,0.285,0.33,0.35,0.37,0.39,0.41,0.43,0.45,0.47,0.49];
ampi = [1;1;0.3;0.1;0.1;0.1;0.1;0.1;0.1;0.1;0.1;0.1];
phai = pi/4*ones(length(omi),1);
%generate the noise-less signal
s = alp*exp(1i*om*(0:N-1)')+exp(1i*(0:N-1)'*omi)*(ampi.*exp(1i*phai));
%%%%%%%%%%%%%%%%%%%%%%%%
%Monte Carlo
%%%%%%%%%%%%%%%%%%%%%%%%
Mmax = floor(N/2);
Nmc = 500;
fb_types = {'DFT','f-ASC','f-APES','fb-ASC','fb-APES','f-MAASC','fb-MAASC'};
Nfb_types = length(fb_types);
a = zeros(Nsnr,Nfb_types,Nmc);
r = zeros(Nsnr,2,Nmc);
p = zeros(Nsnr,2,Nmc);
f = exp(1i*(0:N-1)'*om); % fourier vector

for jj=1:Nsnr
  disp(['Running ' num2str(Nmc) ' MC iterations for an SNR of ' num2str(SNR(jj)) ' dB'])
  for ii=1:Nmc
    %generate new signal
    x = s+sqrt(sig(jj)/2)*(randn(N,1)+1i*randn(N,1));
    %estimate complex amplitudes @ the true frequency
    a(jj,1,ii) = (f'*x)/N;
    a(jj,2,ii) = asc(x,Mmax,om,'unbiased');
    a(jj,3,ii) = apes(x,Mmax,om,'unbiased');
    a(jj,4,ii) = asc(x,Mmax,om,'forback');
    a(jj,5,ii) = apes(x,Mmax,om,'forback');
    [am,rm,pm,~,m] = maasc(x,Mmax,om,'forward');
    r(jj,1,ii) = sum(rm.*m,1);
    p(jj,1,ii) = sum(pm.*m,1);
    a(jj,6,ii) = sum(am.*m,1);
    [am,rm,pm,~,m] = maasc(x,Mmax,om,'forback');
    r(jj,2,ii) = sum(rm.*m,1);
    p(jj,2,ii) = sum(pm.*m,1);
    a(jj,7,ii) = sum(am.*m,1);
  end
end

%% MC results and plots
a_forms = {'Real','Imag','Amp','Phase'};
Na_forms = length(a_forms);

%CRLB
CRLB = zeros(Nsnr,Na_forms);
% CRLB(:,1) = sig'/(N*amp^2)*(real(alp)^2/2+imag(alp)^2*(2*N-1)/(N+1)); % real
% CRLB(:,2) = sig'/(N*amp^2)*(imag(alp)^2/2+real(alp)^2*(2*N-1)/(N+1)); % imag
% CRLB(:,3) = sig'/(2*N); % amplitude
% CRLB(:,4) = sig'*(2*N-1)/(amp^2*N*(N+1)); %phase
CRLB(:,1) = sig'/(2*N); % real
CRLB(:,2) = sig'/(2*N); % imag
CRLB(:,3) = sig'/(2*N); % amplitude
CRLB(:,4) = sig'/(2*N*amp^2); %phase


%bias
Abias = zeros(Nsnr,Nfb_types,Na_forms);
Abias(:,:,1) = sum(real(a)-real(alp),3)/Nmc; % real part
Abias(:,:,2) = sum(imag(a)-imag(alp),3)/Nmc; % imag part
Abias(:,:,3) = sum(abs(a)-amp,3)/Nmc; % amplitude
Abias(:,6:7,3) = sum(r-amp,3)/Nmc; % amplitude
Abias(:,:,4) = sum(angle(a)-phase,3)/Nmc; % phase
Abias(:,6:7,4) = sum(p-phase,3)/Nmc; % amplitude
figure(1)
for ii = 1:Na_forms
  subplot(2,2,ii)
  plot(SNR,Abias(:,:,ii))
  title(a_forms{ii})
  xlabel('SNR [dB]')
  ylabel('Bias')
  legend(fb_types)
end

%variance
Avar = zeros(Nsnr,Nfb_types,Na_forms);
Avar(:,:,1) = var(real(a),0,3); % real part
Avar(:,:,2) = var(imag(a),0,3); % imag part
Avar(:,:,3) = var(abs(a),0,3); % amplitude
Avar(:,6:7,3) = var(r,0,3); % amplitude
Avar(:,:,4) = var(angle(a),0,3); % phase
Avar(:,6:7,4) = var(p,0,3); % phase
figure(2)
for ii = 1:Na_forms
  subplot(2,2,ii)
  semilogy(SNR,[Avar(:,:,ii),CRLB(:,ii)])
  title(a_forms{ii})
  xlabel('SNR [dB]')
  ylabel('Variance')
  legend([fb_types,'CRLB'])
end

%RMSE
Amse = zeros(Nsnr,Nfb_types,Na_forms);
Amse(:,:,1) = sum((real(a)-real(alp)).^2,3)/Nmc; % real part
Amse(:,:,2) = sum((imag(a)-imag(alp)).^2,3)/Nmc; % imag part
Amse(:,:,3) = sum((abs(a)-amp).^2,3)/Nmc; % amplitude
Amse(:,6:7,3) = sum((r-amp).^2,3)/Nmc; % amplitude
Amse(:,:,4) = sum((angle(a)-phase).^2,3)/Nmc; % phase
Amse(:,6:7,4) = sum((p-phase).^2,3)/Nmc; % phase
figure(3)
for ii = 1:Na_forms
  subplot(2,2,ii)
  semilogy(SNR,[Amse(:,:,ii),CRLB(:,ii)])
  title(a_forms{ii})
  xlabel('SNR [dB]')
  ylabel('MSE')
  legend([fb_types,'CRLB'])
end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% export the data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%bias
data_real = [SNR',Abias(:,:,1)];
data_imag = [SNR',Abias(:,:,2)];
data_amp = [SNR',Abias(:,:,3)];
data_pha = [SNR',Abias(:,:,4)];
fid_real = fopen(['bias_real.table'], 'wt');
fid_imag = fopen(['bias_imag.table'], 'wt');
fid_amp = fopen(['bias_amp.table'], 'wt');
fid_pha = fopen(['bias_pha.table'], 'wt');
fprintf(fid_real,'%%Bias for the real part\n');
fprintf(fid_real,'snr dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
fprintf(fid_imag,'%%Bias for the imaginary part\n');
fprintf(fid_imag,'snr dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
fprintf(fid_amp,'%%Bias for the amplitude\n');
fprintf(fid_amp,'snr dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
fprintf(fid_pha,'%%Bias for the phase\n');
fprintf(fid_pha,'snr dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
for ii=1:Nsnr
  fprintf(fid_real,'%1.5f ',data_real(ii,:));
  fprintf(fid_real,'\n');
  fprintf(fid_imag,'%1.5f ',data_imag(ii,:));
  fprintf(fid_imag,'\n');
  fprintf(fid_amp,'%1.5f ',data_amp(ii,:));
  fprintf(fid_amp,'\n');
  fprintf(fid_pha,'%1.5f ',data_pha(ii,:));
  fprintf(fid_pha,'\n');
end
fclose(fid_real);
fclose(fid_imag);
fclose(fid_amp);
fclose(fid_pha);
%variance
data_real = [SNR',CRLB(:,1),Avar(:,:,1)];
data_imag = [SNR',CRLB(:,2),Avar(:,:,2)];
data_amp = [SNR',CRLB(:,3),Avar(:,:,3)];
data_pha = [SNR',CRLB(:,4),Avar(:,:,4)];
fid_real = fopen(['var_real.table'], 'wt');
fid_imag = fopen(['var_imag.table'], 'wt');
fid_amp = fopen(['var_amp.table'], 'wt');
fid_pha = fopen(['var_pha.table'], 'wt');
fprintf(fid_real,'%%Variance for the real part\n');
fprintf(fid_real,'snr crlb dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
fprintf(fid_imag,'%%Variance for the imaginary part\n');
fprintf(fid_imag,'snr crlb dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
fprintf(fid_amp,'%%Variance for the amplitude\n');
fprintf(fid_amp,'snr crlb dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
fprintf(fid_pha,'%%Variance for the phase\n');
fprintf(fid_pha,'snr crlb dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
for ii=1:Nsnr
  fprintf(fid_real,'%1.10f ',data_real(ii,:));
  fprintf(fid_real,'\n');
  fprintf(fid_imag,'%1.10f ',data_imag(ii,:));
  fprintf(fid_imag,'\n');
  fprintf(fid_amp,'%1.10f ',data_amp(ii,:));
  fprintf(fid_amp,'\n');
  fprintf(fid_pha,'%1.10f ',data_pha(ii,:));
  fprintf(fid_pha,'\n');
end
fclose(fid_real);
fclose(fid_imag);
fclose(fid_amp);
fclose(fid_pha);
%mse
data_real = [SNR',CRLB(:,1),Amse(:,:,1)];
data_imag = [SNR',CRLB(:,2),Amse(:,:,2)];
data_amp = [SNR',CRLB(:,3),Amse(:,:,3)];
data_pha = [SNR',CRLB(:,4),Amse(:,:,4)];
fid_real = fopen(['mse_real.table'], 'wt');
fid_imag = fopen(['mse_imag.table'], 'wt');
fid_amp = fopen(['mse_amp.table'], 'wt');
fid_pha = fopen(['mse_pha.table'], 'wt');
fprintf(fid_real,'%%MSE for the real part\n');
fprintf(fid_real,'snr crlb dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
fprintf(fid_imag,'%%MSE for the imaginary part\n');
fprintf(fid_imag,'snr crlb dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
fprintf(fid_amp,'%%MSE for the amplitude\n');
fprintf(fid_amp,'snr crlb dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
fprintf(fid_pha,'%%MSE for the phase\n');
fprintf(fid_pha,'snr crlb dft f_asc f_apes fb_asc fb_apes f_maasc fb_maasc\n');
for ii=1:Nsnr
  fprintf(fid_real,'%1.10f ',data_real(ii,:));
  fprintf(fid_real,'\n');
  fprintf(fid_imag,'%1.10f ',data_imag(ii,:));
  fprintf(fid_imag,'\n');
  fprintf(fid_amp,'%1.10f ',data_amp(ii,:));
  fprintf(fid_amp,'\n');
  fprintf(fid_pha,'%1.10f ',data_pha(ii,:));
  fprintf(fid_pha,'\n');
end
fclose(fid_real);
fclose(fid_imag);
fclose(fid_amp);
fclose(fid_pha);
